@@ -23,6 +23,7 @@ import sys
 import os
 import json
 import re
+import subprocess
 from path import path as p
 
 from mercurial.ui import ui
@@ -31,6 +32,8 @@ from mercurial import encoding
 from mercurial.bookmarks import listbookmarks, readcurrent, pushbookmark
 from mercurial.util import sha1
 from mercurial import hg
+from mercurial.error import RepoLookupError
+
 
 DEBUG_GITIFYHG = os.environ.get("DEBUG_GITIFYHG", "").lower() == "on"
 
@@ -228,6 +231,10 @@ class HGRemote(object):
     def __init__(self, alias, url):
         if hg.islocal(url.encode('utf-8')):
             url = p(url).abspath()
+            # Force git to use an absolute path in the future
+            cmd = ['git', 'config', 'remote.%s.url' % alias, "gitifyhg::%s" % url]
+            subprocess.call(cmd)
+
         gitdir = p(os.environ['GIT_DIR'].decode('utf-8'))
         self.remotedir = gitdir.joinpath('hg', alias)
         self.marks_git_path = self.remotedir.joinpath('marks-git')
@@ -289,7 +296,7 @@ class HGRemote(object):
 
     def do_list(self, parser):
         '''List all references in the mercurial repository. This includes
-        the current head, all branches, and bookmarks.'''
+        the current head, all branches, tags, and bookmarks.'''
 
         current_branch = self.repo.dirstate.branch()
 
@@ -573,6 +580,34 @@ class GitExporter(object):
         from_mark = merge_mark = None
 
         ref = self.parser.line.split()[1]
+        # FIXME: This needs to be in it's own method or function somehow.
+        #        The importer uses similar logic.
+        if ref.startswith('refs/heads/branches/'):
+            branch_name = ref[len('refs/heads/branches/'):]
+            try:
+                tip = self.repo.branchtip(git_to_hg_spaces(branch_name))
+                git_marked_tip = self.marks.tips['branches/%s' % branch_name]
+            except RepoLookupError:
+                # setting these to 0 isn't honest, but it (currently)
+                # only has to pass the if git_marked_tip < tip: below
+                tip = git_marked_tip = 0
+        elif ref == 'refs/heads/master':
+            try:
+                tip = self.repo.branchtip('default')
+                git_marked_tip = self.marks.tips['bookmarks/master']
+            except RepoLookupError:
+                tip = git_marked_tip = 0
+        elif ref.startswith('refs/heads/'):
+            bookmark = ref[len('refs/heads/'):]
+            tip = listbookmarks(self.repo)[git_to_hg_spaces(bookmark)]
+            git_marked_tip = self.marks.tips['bookmarks/%s' % bookmark]
+
+        tip = self.repo[tip].rev()
+        log("%r %r" % (git_marked_tip, tip))
+        if git_marked_tip < tip:
+            output("error %s already exists\n" % ref)
+            sys.exit(1)
+
         commit_mark = self.parser.read_mark()
         author = self.parser.read_author()
         committer = self.parser.read_author()
@@ -622,7 +657,7 @@ class GitExporter(object):
                     files[file] = {'ctx': self.repo[parent_from][file]}
 
         if ref.startswith('refs/heads/branches/'):
-            extra['branch'] = git_to_hg_spaces(ref.rpartition('/')[2])
+            extra['branch'] = git_to_hg_spaces(branch_name)
 
         def get_filectx(repo, memctx, file):
             filespec = files[file]
@@ -663,7 +698,6 @@ def main():
     '''Main entry point for the git-remote-gitifyhg command. Parses sys.argv
     and constructs a parser from the result.
     '''
-    log(repr(sys.argv))
     HGRemote(*[x.decode('utf-8') for x in sys.argv[1:3]]).process()
     try:
         sys.stderr.close()
